package cz.cvut.ok.kapo.paa.batoh.app;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.logging.Level;
import java.util.logging.Logger;

import cz.cvut.ok.kapo.paa.batoh.Solution;
import cz.cvut.ok.kapo.paa.batoh.Statistic;
import cz.cvut.ok.kapo.paa.batoh.engines.FileCruncher;
import cz.cvut.ok.kapo.paa.batoh.utils.ConfigurationException;
import cz.cvut.ok.kapo.paa.batoh.utils.ConfigurationReader;
import cz.cvut.ok.kapo.paa.batoh.utils.SolutionException;
import cz.cvut.ok.kapo.paa.batoh.utils.SolutionsWriter;

/**
 * Projede zadany adresar a pro vsechny obsazene soubory
 * spusti zadany engine.
 * @author karel
 */
public class DirectoryCruncher {
	public final static String LOGGER_NAME = "DIRECTORY_CRUNCHER_LOGGER";
	protected String dirIn, dirOut;
	protected Writer writer;
	public Logger log;
	public static final String [] enginesName = {	
		"cz.cvut.ok.kapo.paa.batoh.engines.bf.BrutalForceEngine",
		"cz.cvut.ok.kapo.paa.batoh.engines.sh.SimpleHeuristicEngine",
		"cz.cvut.ok.kapo.paa.batoh.engines.bab.BranchAndBoundsEngine",
		"cz.cvut.ok.kapo.paa.batoh.engines.bch.BestCostHeuristicEngine",
		"cz.cvut.ok.kapo.paa.batoh.engines.dp.DynamicProgrammingEngine",
		"cz.cvut.ok.kapo.paa.batoh.engines.ch.CombinedHeuristicEngine"
		};
	
	/**
	 * 
	 * @param dirIn	vstupni adresar.
	 * @param dirOut adresar s vystupnimi soubory.
	 * @param writer writer, do ktereho se budou zapisovat statistiky.
	 */
	public DirectoryCruncher(String dirIn, String dirOut, Writer statWriter) {
		super();
		this.dirIn=dirIn;
		this.dirOut=dirOut;
		this.writer=statWriter;
		log = Logger.getLogger(DirectoryCruncher.LOGGER_NAME);
	}
	
	/**
	 * Spusti vypocet pres vsechny soubory v zadanem adreari.
	 *
	 */
	public void execute(){
		File fdi, fdo;
		fdi = new File(dirIn);
		fdo = new File(dirOut);
		
		if (!fdi.isDirectory() | !fdo.isDirectory()){
			log.warning("Input or output directory is not a folder! exitting");
			return;
		}
		FileReader fr;
		Writer fw;
		ConfigurationReader cr;
		SolutionsWriter sw;
		Statistic stats = new Statistic(true);

		try{
			writer.write("pocet;cas[ms];chyba[prumerna];chyba[maximalni]\n\r");
		}
		catch(IOException ex){
			log.log(Level.SEVERE, "Chyba pri zapisu do statistik!", ex);
		}		
		
		for (File fi: fdi.listFiles()){
			try {
				if (fi.isDirectory())
					continue;
				System.out.println("Soubor "+fi.getName());
				//inicializace
				fr = new FileReader(fi);
				cr = new ConfigurationReader(fr);
				try {
					cr.readConfigurations();
				}
				catch (ConfigurationException ce){
					log.log(Level.SEVERE, "Chyba pri nacitani konfigurace!", ce);
					continue;
				}
				
				//novy zpusob zpracovani souboru:
				for (String eName: enginesName){
					String name = eName.substring(eName.lastIndexOf(".")+1, eName.length());
					System.out.println(eName);
					fw = new FileWriter(fdo.getAbsolutePath()+File.separator+name+"_"+fi.getName());
					FileCruncher fc = new FileCruncher(cr.getConfigurations(), eName);
					fc.execute();
					sw = new SolutionsWriter(fw);
					sw.setSolutions(fc.getSolutions());
					sw.writeSolutios();
					fw.close();
					//statistiky: plneni
					for (Solution sol: fc.getSolutions())
						stats.addSolution(sol,eName);
					
					// doplnujici universalni vystup
					fw = new FileWriter(fdo.getAbsolutePath()+File.separator+name+"_MIN"+fi.getName());
					for (Solution solution: fc.getSolutions()){
						fw.write(solution.getConfiguration().getId()
								+" "
								+solution.getStatesCount()
								+" "
								+solution.getSolutionCost()
								+"\r\n");
					}
					fw.close();
				}
				
			}
			catch(IOException ex){
				log.log(Level.SEVERE, "Chyba pri otevirani souboru: "+ fi.getName(), ex);
			}
			catch(SolutionException ex){
				log.log(Level.SEVERE, "Chyba pri ukladani vysledku!", ex);
			}
		}
		//statistiky: tisk
		for (String eName: enginesName){
			String name = eName.substring(eName.lastIndexOf(".")+1, eName.length());
			try{
				fw = new FileWriter(fdo.getAbsolutePath()+File.separator+name+"Stats.csv");
				stats.writeStats(fw,eName);
				fw.close();
			}catch (IOException ioe){
				log.log(Level.SEVERE, "Chyba pri ukladani statistik!", ioe);
			}
		}
		try{
			fw = new FileWriter(fdo.getAbsolutePath()+File.separator+"AllStats.csv");
			stats.writeStatsWithErrors(fw,"cz.cvut.ok.kapo.paa.batoh.engines.bab.BranchAndBoundsEngine");
			fw.close();
		}catch (IOException ioe){
			log.log(Level.SEVERE, "Chyba pri ukladani souhrnnych statistik!", ioe);
		}


	}
	

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		FileWriter sw=null;
		DirectoryCruncher dc=null;
		
		if (args.length<2){
			System.out.println("Chybi parametry: <adresar.vstupni> [adresar.vystupni] <stats.soubor>\n\r");
			return;
		}
		else if (args.length<3){
			try {
				sw = new FileWriter(args[1]);
			}
			catch(IOException ex){
				System.out.println("Chyba pri otevirani souboru se statistikami!");
			}
			dc=new DirectoryCruncher(args[0],args[0],sw);
			dc.execute();
		}
		else if (args.length<4){
			try {
				sw = new FileWriter(args[2]);
			}
			catch(IOException ex){
				System.out.println("Chyba pri otevirani souboru se statistikami!");
			}
			dc=new DirectoryCruncher(args[0],args[1],sw);
			dc.execute();
		}

		System.out.println("Hotovo");

	}

}
